export declare type PluggablePeriod = 'pre' | 'post';
export declare type PluggablePeriodSet = PluggablePeriod | 'pre post';
export declare type PluggableHooks = {
    pre: {
        [key: string]: Function[];
    };
    post: {
        [key: string]: Function[];
    };
};
export default class Pluggable {
    static _initialized: boolean;
    static _hooks: PluggableHooks;
    static _declares: {
        path: string;
        value: any;
    }[];
    static _subscribers: Pluggable[];
    private _hooks;
    [key: string]: any;
    /**
     *
     * @param {typeof Pluggable} pluggable
     * @private
     */
    private static _initialize(pluggable);
    /**
     * @constructor
     * @public
     */
    constructor();
    /**
     *
     * @param {PluggablePeriod} period
     * @param {string | string[]} name
     * @param {Function} funcs
     * @private
     */
    private _addHooks(period, name, ...funcs);
    /**
     *
     * @param {string | string[]} name
     * @param {Function} funcs
     * @public
     */
    pre(name: string | string[], ...funcs: Function[]): void;
    /**
     *
     * @param {string | string[]} name
     * @param {Function} funcs
     * @public
     */
    post(name: string | string[], ...funcs: Function[]): void;
    /**
     *
     * @param {PluggablePeriod} period
     * @param {string} name
     * @param {...*}argus
     * @return {Promise<void>}
     * @private
     */
    private _triggerHooks(period, name, ...argus);
    /**
     *
     * @param {string | string[]} name
     * @param {Function} funcs
     * @public
     */
    static pre(name: string | string[], ...funcs: Function[]): void;
    /**
     *
     * @param {string | string[]} name
     * @param {Function} funcs
     */
    static post(name: string | string[], ...funcs: Function[]): void;
    /**
     *
     * @param {PluggablePeriod} period
     * @param {string | string[]} name
     * @param {Function} funcs
     * @private
     */
    private static _addHooks(period, name, ...funcs);
    /**
     *
     * @param {string} path
     * @param value
     */
    static declare(path: string, value: any): void;
    /**
     *
     * @param {string} funcName
     * @param {string} hookName
     * @param {PluggablePeriodSet} periodsSet
     */
    static wrapHook(funcName: string, hookName: string, periodsSet: PluggablePeriodSet): void;
    /**
     *
     * @param {string} funcAndHookName
     * @param {PluggablePeriodSet} periodsSet
     * @param  _
     */
    static wrapHook(funcAndHookName: string, periodsSet: PluggablePeriodSet, _: never): void;
}
export interface DefaultPluggin<T> {
    (param: T): Promise<T>;
}
